package tfg.fuzzy.primitives.creation;

import org.nlogo.api.Argument;
import org.nlogo.api.Context;
import org.nlogo.api.DefaultReporter;
import org.nlogo.api.ExtensionException;
import org.nlogo.api.LogoException;
import org.nlogo.api.LogoList;
import org.nlogo.api.Syntax;

import tfg.fuzzy.general.SupportFunctions;
import tfg.fuzzy.sets.general.FuzzySet;
import tfg.fuzzy.sets.points.IntervalWithPointsSet;

/**
 * This class creates a new interval with points set with the label given.
 * Implements the primitive "interval-with-points-set-with-label".
 * 
 * @author Marcos Almendres.
 *
 */
public class IntervalWithLabel extends DefaultReporter {

	/**
	 * This method tells Netlogo the appropriate syntax of the primitive.
	 * Receives a string and a list and returns a Wildcard.
	 */
	public Syntax getSyntax() {
		return Syntax.reporterSyntax(
				new int[] { Syntax.StringType(), Syntax.ListType() },
				Syntax.WildcardType());
	}

	/**
	 * This method respond to the call from Netlogo and returns the set.
	 * 
	 * @param arg0
	 *            Arguments from Netlogo call, in this case a string and a list.
	 * @param arg1
	 *            Context of Netlogo when the call was done.
	 * @return A new IntervalWithPointsSet.
	 */
	@Override
	public Object report(Argument[] arg0, Context arg1)
			throws ExtensionException, LogoException {
		// The same as Interval
		LogoList params = arg0[1].getList();
		LogoList paramsDef = (LogoList) params.get(0);
		LogoList paramsPoints = (LogoList) params.get(1);
		double[] universe = SupportFunctions.IWPFormat(params);
		FuzzySet createdSet = new IntervalWithPointsSet(
				SupportFunctions.checkListFormat(paramsPoints), true,
				arg0[0].getString(), universe, (Double) paramsDef.get(1));
		// Add the set to a registry, allowing to look for it in the future.
		SupportFunctions.addToRegistry(createdSet, arg0[0].getString(),arg1);
		return createdSet;
	}
}
